@@ -10,7 +10,7 @@ use File::Spec;
 use XML::LibXML;
 use XML::LibXSLT;
 
-use XML::Grammar::Fortune::ConfigData;
+use File::ShareDir ':ALL';
 
 use base 'Class::Accessor';
 
@@ -30,11 +30,11 @@ XML::Grammar::Fortune - convert the FortunesXML grammar to other formats and fro
 
 =head1 VERSION
 
-Version 0.0300
+Version 0.0400
 
 =cut
 
-our $VERSION = '0.0300';
+our $VERSION = '0.0400';
 
 
 =head1 SYNOPSIS
@@ -167,8 +167,7 @@ sub _init
 
     $self->_output_mode($args->{output_mode} || "filename");
 
-    my $data_dir = $args->{'data_dir'} ||
-        XML::Grammar::Fortune::ConfigData->config('extradata_install_path')->[0];
+    my $data_dir = $args->{'data_dir'} || dist_dir( 'XML-Grammar-Fortune' );
 
     $self->_data_dir($data_dir);
 
